﻿using EvoPdf;
using EvoPdf.PdfPrint;
using Fleck;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Drawing.Printing;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace BergenSilentPrinting
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        string url;
        string contentType = "";
        private IWebSocketConnection socket;
        private WebSocketServer server;
        string filePath = "C:/Windows/Temp/Bergen_Document_Print.pdf".Trim();
        PdfPrint pdfPrint;
        HtmlToPdfConverter htmlToPdfConverter;

        public MainWindow()
        {
            InitializeComponent();
            WebSocketServer();

        }

        void WebSocketServer()
        {
            try
            {
                server = new WebSocketServer("ws://0.0.0.0:8080");
                server.Start(sckt => {
                    socket = sckt;
                    sckt.OnOpen = () => OnOpen();
                    sckt.OnClose = () => OnClose();
                    sckt.OnMessage = message => OnMessage(message);
                });
            }
            catch (Exception ex)
            {
                MessageBox.Show(string.Format("An error occurred. {0}", ex.Message), "Error");
            }
        }

        private void OnMessage(string message)
        {
            try
            {
                RequestDto req = JsonConvert.DeserializeObject<RequestDto>(message);
                RequestDto req1 = JsonConvert.DeserializeObject<RequestDto>(req.Data);

                PrintNow(req1.Url, req1.Printer);

            }
            catch (Exception ex)
            {
                socket.Send(JsonConvert.SerializeObject(new RequestDto() { Type = "Error", Data = ex.Message.ToString() }).ToString());
            }
        }

        private void PrintNow(string url_path, string printer_name)
        {

            url = url_path;
            try
            {

                HttpWebRequest request = HttpWebRequest.Create(url) as HttpWebRequest;
                if (request.Equals(string.Empty))
                {
                    socket.Send("No Request");
                }
                else
                {
                    HttpWebResponse response = request.GetResponse() as HttpWebResponse;



                    if (response.Equals(string.Empty))
                    {
                        socket.Send("No response");
                    }
                    else
                    {
                        contentType = response.ContentType;
                        response.Dispose();
                    }
                }
            }

           catch(Exception e)
            {
                
            }

            if (contentType.Equals("application/pdf"))
            {

                try
                {
                    WebClient wc = new WebClient();
                    wc.DownloadFile(url, filePath);
                }
                catch(Exception e)
                {
                    
                }
            }
            else
            {
                htmlToPdfConverter = new HtmlToPdfConverter();
                htmlToPdfConverter.LicenseKey = "TsDTwdTRwdXW0cHWz9HB0tDP0NPP2NjY2MHR";
                htmlToPdfConverter.ConversionDelay = 2;
                htmlToPdfConverter.HtmlViewerWidth = 800;
                htmlToPdfConverter.PdfDocumentOptions.EmbedFonts = true;
                htmlToPdfConverter.HtmlViewerZoom = 105;

                htmlToPdfConverter.PdfDocumentOptions.PdfPageSize = EvoPdf.PdfPageSize.Letter;
                htmlToPdfConverter.PdfDocumentOptions.PdfPageOrientation = EvoPdf.PdfPageOrientation.Portrait;
                htmlToPdfConverter.PdfDocumentOptions.LeftMargin = 0;
                htmlToPdfConverter.PdfDocumentOptions.RightMargin = 0;
                htmlToPdfConverter.PdfDocumentOptions.TopMargin = 0;
                htmlToPdfConverter.PdfDocumentOptions.BottomMargin = 0;
                htmlToPdfConverter.PdfDocumentOptions.SinglePage = false;
                htmlToPdfConverter.ConvertUrlToFile(url, filePath);
                htmlToPdfConverter = null;
            }






            if (filePath.Trim().Equals(string.Empty))
            {
                
                return;
            }


            // create the PDF printer 
            pdfPrint = new PdfPrint();
            pdfPrint.DefaultPageSettings.Margins = new System.Drawing.Printing.Margins(36, 36, 36, 36);

            // set the license key
            pdfPrint.LicenseKey = "TsDTwdTRwdXW0cHWz9HB0tDP0NPP2NjY2MHR";

            // set the document name
            pdfPrint.DocumentName = "Bergen_Document_Print";

            // select the printer
            pdfPrint.Resolution = 300;
            pdfPrint.ShowStatusDialog = false;


            string selectedPrinterName = printer_name;

            if (selectedPrinterName.Equals(string.Empty))
            {
                PrinterSettings default_printer = new PrinterSettings();
                string default_printer_name = default_printer.PrinterName;
                pdfPrint.PrinterSettings.PrinterName = default_printer_name;
                
            }
            else
            {
                pdfPrint.PrinterSettings.PrinterName.ToLower().Contains(selectedPrinterName.ToLower());
                
            }


            try
            {
                pdfPrint.Print(filePath);
                socket.Send("done");
                pdfPrint = null;
            }
            catch (Exception ex)
            {
                // The conversion failed
                
                return;
            }
        }

        private void OnClose()
        {
            try
            {

            }
            catch (Exception ex)
            {
                
            }
        }

        private void OnOpen()
        {
            try
            {

            }
            catch (Exception ex)
            {
                
            }
        }
    }

    public class RequestDto
    {
        public string Type { get; set; }
        public string Data { get; set; }
        public string Url { get; set; }
        public string Printer { get; set; }
    }


}
